/**
 * Copyright Qnext Inc. 2002-2018
 * http://www.qnext.com
 * Date: 24.7.2018
 * Time: 18:11
 * Author Petar Vanchev (petarv@qnext.com)
 */

/**
 *
 * @param {jQuery} $container
 * @constructor
 */
var CompAntivirusSettings = function ($container) {
    BaseComp.call(this, $container);
    /**
     *
     * @type {Array}
     * @private
     */
    this._avList = [AntivirusInfo.CUSTOM, AntivirusInfo.NONE];

    this._initContent();
};

QnUtil.inherits(CompAntivirusSettings, BaseComp);
QnUtil.extend(CompAntivirusSettings.prototype, {

    /* Public method */

    /**
     *
     * @param {AntivirusInfo[]} [avList]
     */
    updatePanel: function (avList) {
        if(avList) {
            this._avList = avList;
            this._renderAvsList();
        }
        this._updateSelectionInAvsList();
        var $comp = this._getCont().find("#CompAntivirusSetting");
        setTextContext("antivirus_settings_info");
        // add focus handler
        $comp.on("focus", "input", function () {
            switchTT($comp.parent());
        });
        // add hover handler
        $comp.hover(function () {
            switchTT($comp.parent());
        }, function () {
            $("body").find(".tooltipCont").css("display", "none");
            $(':focus').focus();
        });
    },

    /**
     * Get currently selected antivirus from list with detected
     *
     * @return {AntivirusInfo} can be null if currently selected one from list is same as this get
     *                  from globals {@link KEY_GLOBAL_SELECTED_ANTIVIRUS}.
     *                  Will be always valid, if {@link AntivirusConfig#isNotSet}=true.
     */
    getSelectedAv: function () {
        var avInfo = null;
        var hasChange = false;
        var avConfig = getGlobal(KEY_GLOBAL_SELECTED_ANTIVIRUS);
        var /*AntivirusInfo*/selectedAvInfo = avConfig.getAvInfo();
        var selectedAvid = selectedAvInfo.getAvid();
        var selectedAvCmd = selectedAvInfo.getAvCmd();
        var avid = $('select[name=avs_select]').val();
        var avCmd = this._getCont().find(".filed_avs_command_line").val();
        // Always allow to change AV explicitly it is not set yet (isNotSet)
        if ((selectedAvid && selectedAvid !== avid) || (selectedAvCmd !== avCmd) || avConfig.isNotSet())
            hasChange = true;

        if(hasChange) {
            for (var i = 0; i < this._avList.length; i++) {
                var info = this._avList[i];
                if (info.getAvid() === avid) {
                    avInfo = info.cloneWithCmd(avCmd);
                    break;
                }
            }
        }
        if (isLogDbg())
            logDbg("Get selected AV: " + avInfo);

        return avInfo;
    },

    /**
     * Enable/Disable radio buttons
     *
     * @param {Boolean} [onDisable]
     */
    setControlsState: function (onDisable) {
        this._getCont().find("input").attr("disabled", onDisable);
        $('select[name=avs_select]').attr("disabled", onDisable);
    },


    /* Protected method */

    _initContent: function () {
        var $comp = this.createCont().attr("id", "CompAntivirusSetting").addClass("CompAntivirusSetting").appendTo(this._getCont());
        this.createCont().addClass("antivirusListCont").appendTo($comp);
        this.createCont().addClass("antivirusAdvancedCont").appendTo($comp);
        // render tooltip
        this._renderTooltip();
        // render list with detected av
        this._renderAvsList();
        // render advanced settings
        this._renderAdvanced();
        // hide advanced by default
        this._toggleAdvanced(false);

        this._updateSelectionInAvsList();
    },


    /* Private method */

    /**
     * Add general tooltip for encryption comp that will be visible if any component is on focus or is hovered
     *
     * @private
     */
    _renderTooltip: function () {
        var $container = this._getCont();
        this._addTooltip(this._getCont(), "AntivirusSetting", "antivirus_settings");
    },

    /**
     * Render list with detected antivirus
     *
     * @private
     */
    _renderAvsList: function () {
        var $cont = this._getCont().find(".antivirusListCont");
        // empty container
        $cont.empty();
        // render header
        this.createCont().addClass("control-label")
            .setTextById("antivirus_list_header")
            .appendTo($cont);
        // render list
        var $comp = this.createCont().addClass("form-group antivirusChoice").appendTo($cont);
        this.createCont()
            .addClass("control-label")
            .setTextById("antivirus_list_label")
            .appendTo($comp);
        var $select = $('<select name="avs_select">').addClass("form-control field_avs_select_input").appendTo($comp);
        for (var i = 0; i < this._avList.length; i++) {
            var /*AntivirusInfo*/avInfo = this._avList[i];
            var avid = avInfo.getAvid();
            var avName = avInfo.getAvName();
            var attributes = {
                id: "avs_select_" + avid,
                value: avid
            };

            var $opt = $('<option></option>').text(avName).appendTo($select);

            // Comment because need to show None options in case of MSP
            // See https://jira2.cnexus.com/browse/FF-3782
            // var isHidden = false;
            // switch (avid) {
            //     case AntivirusInfo.NONE_ID:
            //         isHidden = !!getGlobal(KEY_GLOBAL_AV_OBLIGATORY);
            //         break;
            // }
            //
            // if(isHidden)
            //     attributes['hidden'] = true;
            $opt.attr(attributes);
        }

        var self = this;
        // render change selected link
        var $toggleList = this.createCont()
            .addClass("control-label linkLabel")
            .setTextById("antivirus_toggle_advanced_header")
            .appendTo($cont);
        $toggleList.on("click", function () {
            self._toggleAdvanced();
        });

        // handle change of select
        $select.on("change", function () {
            var selected = $(this).val();
            for (var i = 0; i < self._avList.length; i++) {
                var avInfo = self._avList[i];
                if(avInfo.getAvid() === selected) {
                    self._updateAdvanced(avInfo);
                }
            }
        });
    },

    /**
     * Render currently selected antivirus
     *
     * @private
     */
    _renderAdvanced: function () {
        var avCur = this._getAvUsed();
        var $cont = this._getCont().find(".antivirusAdvancedCont");
        // render header
        this.createCont()
            .addClass("control-label")
            .setTextById("antivirus_advanced_settings_header")
            .appendTo($cont);
        //
        var $row = this.createCont().addClass("form-group avs_command_line").appendTo($cont);
        this.createCont()
            .addClass("control-label")
            .setTextById("antivirus_command_line_label")
            .appendTo($row);
        var $input = $('<input class="form-control filed_avs_command_line" required />')
            .setTooltip("field_antivirus_advanced_settings_command_line_placeholder")
            .appendTo($row);
        // render selected
        for (var i = 0; i < this._avList.length; i++) {
            var /*AntivirusInfo*/avInfo = this._avList[i];
            var avid = avInfo.getAvid();
            if(avid === avCur.getAvid()) {
                $input.val(avCur.getAvCmd());
                break;
            }
        }
    },

    /**
     *
     * @param {AntivirusInfo} avInfo
     * @private
     */
    _updateAdvanced: function (avInfo) {
        var cmdVal = avInfo.getAvCmd();
        var selectedAvId = avInfo.getAvid();
        var usedAv = this._getAvUsed();
        if(selectedAvId === usedAv.getAvid())
            cmdVal = usedAv.getAvCmd();

        // disable cmd filed in case when selectedAvId is AntivirusInfo.NONE_ID
        var $cmdLine = this._getCont().find(".filed_avs_command_line");
        $cmdLine.val(cmdVal);

        // show/hide advanced setting link and input. Also enable/disable input
        switch (selectedAvId) {
            case AntivirusInfo.NONE_ID:
                this._toggleAdvanced(false);
                this._toggleAdvancedLink(false);
                $cmdLine.attr("disabled", true);
                break;
            case AntivirusInfo.CUSTOM_ID:
                this._toggleAdvanced(true);
                this._toggleAdvancedLink(true);
                $cmdLine.removeAttr("disabled");
                break;
            default:
                this._toggleAdvancedLink(true);
                $cmdLine.removeAttr("disabled");
                break;
        }
    },

    /**
     * Toggle advanced antivirus settings.
     *
     * @param {Boolean} [isVisible]
     * @private
     */
    _toggleAdvanced: function (isVisible) {
        if(QnUtil.isDefined()) {
            // normalize
            isVisible = !!isVisible;
        }
        this._getCont().find(".antivirusAdvancedCont").toggle(isVisible);
    },

    /**
     * Toggle link that sow/hide advanced antivirus settings.
     *
     * @param {Boolean} [isVisible]
     * @private
     */
    _toggleAdvancedLink: function (isVisible) {
        if(QnUtil.isDefined()) {
            // normalize
            isVisible = !!isVisible;
        }
        this._getCont().find(".linkLabel").toggle(isVisible);
    },

    /**
     * @private
     */
    _updateSelectionInAvsList: function () {
        // default config
        var defAvConfig = new AntivirusConfig(AntivirusInfo.NONE, false, null, true);
        // used config
        var usedAvInfo = this._getAvUsed();
        var usedAvId = usedAvInfo.getAvid();

        var selectInfo = null;
        // check did used av id is found in list with predefined option
        var $selectComp = this._getCont().find('.field_avs_select_input');
        $selectComp.find('option').each(function () {
            var curr = $(this).val();
            if (curr === usedAvId)
                selectInfo = usedAvInfo;
        });
        // fallback to default
        var notFound = false;
        if(!selectInfo) {
            notFound = true;
            selectInfo = defAvConfig.getAvInfo();
        }

        var selectAvId = selectInfo.getAvid();
        // update current dropdown selection
        $selectComp.val(selectAvId).trigger("change");

        // check did current used (selected) av config is valid and show/hide error messages
        var $msgCont = $("." + FrameAntivirusSettings.VID).find("#updatemsgForm");
        var error = this._checkIsValidSelectedAv(selectAvId);
        if(!error) {
            // clear all errors
            clearMessage(ERROR_NO_ANTIVIRUS, $msgCont);
            clearMessage(undefined, $msgCont);
            if(notFound && !defAvConfig.isNotSet())
                showMessage(ERROR_NO_ANTIVIRUS, "antivirus_settings_invalid_configuration", $msgCont);
        } else {
            showMessage(ERROR_NO_ANTIVIRUS, error, $msgCont);
        }
    },

    /**
     * Get did returned as set passive antivirus config is valid one
     * and did antivirus id get from it is found in list with detected one
     *
     * @param {String} avid antivirus id one of {@link AntivirusInfo#NOT_SET_ID} ...
     * @return {boolean} always valid
     * @private
     */
    _checkIsValidSelectedAv: function (avid) {
        var errorStr = null;
        var isAvObl = getGlobal(KEY_GLOBAL_AV_OBLIGATORY);

        var notFound = true;
        //1. check did currently selected avid is in list with detected
        for (var i = 0; i < this._avList.length; i++) {
            var predefAvInfo = this._avList[i];
            if (avid === predefAvInfo.getAvid()) {
                notFound = false;
                break;
            }
        }

        if(notFound)
            errorStr = "antivirus_settings_invalid_configuration";

        if(null === errorStr) {
            // 2. check did config is valid at all
            var avConfig = getGlobal(KEY_GLOBAL_SELECTED_ANTIVIRUS);
            if(avid !== AntivirusInfo.NONE_ID && !avConfig.isConfigProper()) {
                // case when not set and no default
                if(avid === AntivirusInfo.NOT_SET_ID) {
                    if(isAvObl)
                        errorStr = "antivirus_not_selected";
                } else {
                    errorStr = "error_antivirus_invalid";
                }
            }
        }

        if(null == errorStr && isAvObl && avid === AntivirusInfo.NONE_ID)
            errorStr = "antivirus_not_selected";

        return errorStr;
    },

    /**
     * Get current real AV which is used, replacing selected AV ({@link KEY_GLOBAL_SELECTED_ANTIVIRUS})
     * with default AV if selected one is not set ({@link #isNotSet} = true).
     * See FF-2525.
     *
     * @return {AntivirusInfo} Will be valid.
     * @private
     */
    _getAvUsed: function () {
        var /*AntivirusConfig*/ avCon = getGlobal(KEY_GLOBAL_SELECTED_ANTIVIRUS);
        if (avCon.isNotSet()) {
            // If AV was not set yet, we will use default
            // So first search for default
            var avDef = null;
            for (var i = 0, l = this._avList.length; i < l; i++) {
                var /*AntivirusInfo*/av = this._avList[i];
                if (av.isDefault()) {
                    avDef = av;
                    break;
                }
            }
            // If there is a default - replace KEY_GLOBAL_SELECTED_ANTIVIRUS with default one
            if (avDef) {
                if (isLogInfo())
                    logInfo("Use Default instead NotSet as selected AV: " + avDef);
                return avDef;
            } else {
                // case when no default antivirus is found in list and current one is not set. Fallback to none
                if (isLogInfo())
                    logInfo("Use None instead NotSet as selected AV.");
                return AntivirusInfo.NONE;
            }
        }
        return avCon.getAvInfo();
    }


});
