/**
 * Copyright Qnext Inc. 2002-2018
 * http://www.qnext.com
 * Date: 9.2.2018
 * Time: 11:43
 * Author Petar Vanchev (petarv@qnext.com)
 * Author Milen Dimitrov (milend@qnext.com)
 */

/**
 * Activation frame
 *
 * @constructor
 */
var FrameActivate = function () {
    var options = {};
    options["canSendLogs"] = true;
    options["hasHashListener"] = true;

    FrameEncrypt.call(this, FrameActivate.VID, $('body'), options);
};

QnUtil.inherits(FrameActivate, FrameEncrypt);
QnUtil.extend(FrameActivate.prototype, {

    /* Public method */

    updateFrame: function () {
        var hash = location.hash && location.hash.charAt(0) === "#" ? location.hash.substring(1) : null;
        var page;
        if (hash) {
            // If there is a hash - use it
            page = hash;
        } else if (getGlobal(KEY_GLOBAL_CODE_ACTIVATE)) {
            // If no hash - check is there code, then use it
            page = FrameActivate._PAGE_ACTIVATE;
        } else {
            // Finally fallback to home page
            page = FrameActivate._PAGE_HOME;
        }
        log("Change page: " + page);
        var isActivate = page === FrameActivate._PAGE_ACTIVATE;
        $("#pageHome").toggle(!isActivate);
        $("#pageActivate").toggle(isActivate);
        if (isActivate)
            this._startActivate();

        // home page
        setTextContext("l2");
        setTextContext("l14");
        // set tooltip text to activate page
        setTextContext("field_code_info");

        FrameActivate.superclass.updateFrame.apply(this);
    },

    /* Protected method */

    /**
     *
     * @inheritDoc
     * @protected
     */
    _getFieldIdList: function () {
        var list = FrameActivate.superclass._getFieldIdList.call(this);
        list.push(FrameWithForm.FIELD_CODE);
        return list;
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _getBtnIdList: function () {
        return [BTN_ACTIVATE_ID, BTN_CANCEL_ID];
    },

    /**
     *
     * @inheritDoc
     * @private
     */
    _checkCanSubmitImpl: function () {
        if(!this._isAvsDetected())
            return false;

        if(this._hasEncComp())
            return this._compEncryption.checkEncryption(true);

        return FrameActivate.superclass._checkCanSubmitImpl.apply(this);
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _handleSubmitImpl: function (event) {
        var code = this._getFormControl(FrameWithForm.FIELD_CODE).val();
        var self = this;
        var callback = function (msg) {
            self._setControlsState(false);
            showMessage(ERROR_INVALID_CODE, msg);
        };

        return this._validateCode(code, callback);
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _fillSubmitParams: function (params) {
        FrameActivate.superclass._fillSubmitParams.call(this, params);
        params["cmd"] = "add";
        params["user"] = this._getFormControl(FrameWithForm.FIELD_USERNAME).val().trim();
        params["user-password"] = this._getFormControl(FrameWithForm.FIELD_PASSWORD).val().trim();
        params["user-domain"] = this._getFormControl(FrameWithForm.FIELD_DOMAIN).val().trim();
        params["code"] = this._getFormControl(FrameWithForm.FIELD_CODE).val().trim();

        var mspId = getGlobal(KEY_GLOBAL_MSP_ID);
        if (mspId)
            params["mspId"] = mspId;
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _handleSubmitSuccessImpl: function (resp, requestParams) {
        if (resp.result) {
            this._handleActivationSuccess(resp["redirectURL"]);
            log("RegisterCode success with " + requestParams["code"]);
        } else {
            log("RegisterCode failure with " + requestParams["code"]);
            if(this._hasEncComp())
                this._compEncryption.updateEncComps();
        }
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _handleSubmitFailImpl: function (xhr, err) {
        if (xhr.responseText) {
            try {
                var response = JSON.parse(xhr.responseText);
                var error = response["error"];
                log("Code register failed error code: " + error["code"]);
            } catch (e) {
                log("Code register failed: " + e);
            }
        }
        if(this._hasEncComp())
            this._compEncryption.updateEncComps();
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _handleActionImpl: function(action, event) {
        switch (action) {
            case "openRegister":
                this._onRegister();
                break;
            case "openActivate":
                this._onActivate();
                break;
            default:
                FrameActivate.superclass._handleActionImpl.call(this, action, event);
        }
    },

    /**
     * @inheritDoc
     * @protected
     */
    _initContentImpl: function ($cont) {
        document.title = QnLocale.getStr("activate_app_title");
        this._renderPageHome($cont);
        var  $formCont = $('<div id="pageActivate" style="display: none;"></div>').appendTo($cont);
        FrameActivate.superclass._initContentImpl.call(this, $formCont);
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _renderPageTitle: function ($cont) {
        // TODO Milen: For Petar/Iliya: Why there is so complicated structure
        $('<h2 class="secureonly"></h2>')
            .setTextById("activation_title")
            .appendTo($cont);
        var $p = $('<p class="p secureonly">').appendTo($cont);
        $('<span></span>')
            .setTextById("activation_subtitle")
            .appendTo($p);
        // See FF-4442
        $('<p></p>')
            .setTextById("activation_subtitle2")
            .appendTo($cont);
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _setControlsState: function (onDisable, notForSubmit) {
        FrameActivate.superclass._setControlsState.call(this, onDisable, notForSubmit);

        // update input
        this._getFormControl(FrameWithForm.FIELD_CODE, true).attr("disabled", onDisable);
        // update buttons
        var $btnSubmit = this._getBtn(BTN_ACTIVATE_ID);
        if(!notForSubmit)
            $btnSubmit.button(onDisable ? 'loading' : 'reset');
        $btnSubmit.attr("disabled", onDisable);
    },

    /* Private method */

    /**
     *
     * @param {jQuery} $cont
     * @private
     */
    _renderPageHome: function ($cont) {
        var $comp = $('<div id="pageHome">' +
            '<div class="homeTitleCont">' +
            '<h2 class="aliasLogo"></h2>' +
            '<a class="button buttonRegister" href="#" target="_blank"></a>' +
            '</div>' +
            '<h3 data-i18n="l1"></h3>' +
            '<p class="p" data-i18n="l2" id="l2"></p>' +
            '<p class="p" data-i18n="l3"></p>' +
            '<ul>' +
            '<li data-i18n="l4"></li>' +
            '</ul>' +
            '<p class="p" data-i18n="l5"></p>' +
            '<ul>' +
            '<li data-i18n="l6"></li>' +
            '<li data-i18n="l7"></li>' +
            '<li data-i18n="l8"></li>' +
            '</ul>' +
            '<p class="p" data-i18n="l9"></p>' +
            '<ul>' +
            '<li data-i18n="l10"></li>' +
            '<li data-i18n="l11"></li>' +
            '<li data-i18n="l12"></li>' +
            '</ul>' +
            '<h3 data-i18n="l13"></h3>' +
            '<ul>' +
            '<li id="l14" data-i18n="l14"></li>' +
            '<li data-i18n="l15"></li>' +
            '</ul>' +
            '<h3 data-i18n="l16"></h3>' +
            '<span data-i18n="l17"></span> ' +
            '<span><a data-i18n="l18" href="http://help.fileflex.com" target="_blank"></a></span>' +
            ' <span data-i18n="l19"></span>' +
            '<p class="p">' +
            '<span data-i18n="l20"></span> ' +
            '<span><a id="actionActivate" data-i18n="l21" href="#activate"></a></span>' +
            '</p>' +
            '</div>');
        $comp.find(".aliasLogo").setTextById("activation_title");
        // Milen: Not good, but structure above is too complex to repeat
        QnLocale.setTextFromAttr($comp);

        $comp.appendTo($cont);

        var self = this;
        $comp.find(".buttonRegister").setTextById("home_register_button")
            .on("click", function (e) {
                self._handleAction("openRegister", e);
        });
        $comp.find("#actionActivate").on("click", function (e) {
            self._handleAction("openActivate", e);
        });

        // Hide initially both pages until languages are loaded
        $comp.hide();
    },

    /**
     *
     * @private
     */
    _onRegister: function () {
        var params = {
            "type" : "canvas"
        };
        var registerUrl = "https://share.fileflex.com/fbweb/app/public/view/register";
        var deviceScheme = getGlobal(KEY_GLOBAL_DEVICE_SCHEME);
        if(deviceScheme)
            params["alias"] = deviceScheme;

        registerUrl = addParams(registerUrl, params);

        window.open(registerUrl,'_blank');
    },

    /**
     *
     * @private
     */
    _onActivate: function () {
        var activateUrl = window.location.href;
        // var deviceScheme = getGlobal(KEY_GLOBAL_DEVICE_SCHEME);
        // if (deviceScheme)
        //     activateUrl = addParams(activateUrl, {"alias" : deviceScheme});

        activateUrl += "#activate";

        window.open(activateUrl, '_self');
    },

    /**
     *
     * @private
     */
    _startActivate: function () {
        // enter a username into the edit box
        var username = getGlobal(KEY_GLOBAL_USERNAME);
        if (username) {
            log("found username: " + username);
            this._getFormControl(FrameWithForm.FIELD_USERNAME).val(username);
        }

        // set the domain if passive provided one (set in the settings page)
        var defDomain =  getGlobal(KEY_GLOBAL_DEFAULT_DOMAIN);
        if (defDomain) {
            log("found default domain: " + defDomain);
            this._getFormControl(FrameWithForm.FIELD_DOMAIN).val(defDomain);
        }

        // enter a domain into the edit box
        var domain = getGlobal(KEY_GLOBAL_DOMAIN);
        if (domain) {
            log("found domain: " + domain);
            this._getFormControl(FrameWithForm.FIELD_DOMAIN).val(domain);
        }

        // enter a code into the edit box
        var code = getGlobal(KEY_GLOBAL_CODE_ACTIVATE);
        if (code) {
            log("found activation code: " + code);
            this._getFormControl(FrameWithForm.FIELD_CODE).val(code);
        }

        var retryLink = getGlobal(KEY_GLOBAL_RETRY_LINK);
        if (retryLink)
            log("found retryLink: " + retryLink);
    },

    /**
     * Validate activation code
     *
     * @param {String} code
     * @param {Function} onFailure
     * @return {Boolean} returns an object containing status:true/false
     * @private
     */
    _validateCode: function(code, onFailure) {
        if (code.length < 4) {
            onFailure(QnLocale.getStr("error_code_short"));
            return false;
        }
        return true;
    },

    /**
     * Handle successful response
     *
     * @param {String} redirectURL
     * @private
     */
    _handleActivationSuccess: function (redirectURL) {
        // success
        log("activation successful");
        // indicate success visually
        showMessage(ERROR_OK, "activate_error_ok");
        this._getFormControl(FrameWithForm.FIELD_CODE).val("");

        // if a redirect URL was provided by passive (via the server) we use it to send the user back immediately
        if (redirectURL !== null) {
            // if there is a redirect URL, we immediately redirect the user
            log("redirectURL found: " + redirectURL);
            window.location.href = redirectURL;
        } else {
            log("no redirect URL was found");
        }
    },

    /**
     * Check did antivirus scanner is detected or not
     *
     * @return {Boolean} always valid
     * @private
     */
    _isAvsDetected: function () {
        // Milen: Last requirements: Passive should report the error,
        // So I have commented this and always returns true.
        // we cannot detect that the default one will be used because we do not call get-antivirus-list.
        // And we cannot call because it is under secure domain.
        // See FF-2525
        return true;
    }

});

/**
 *
 * @type {string}
 * @constant
 */
FrameActivate.VID = "FrameActivate";

/**
 *
 * @type {string}
 * @private
 * @constant
 */
FrameActivate._PAGE_ACTIVATE = "activate";
/**
 *
 * @type {string}
 * @private
 * @constant
 */
FrameActivate._PAGE_HOME = "home";
